// Models of backward masking -- July 2002
// Greg Francis (gfrancis@psych.purdue.edu)
// Purdue University
// This material is based upon work supported by the National Science Foundation under Grant No. 0108905. 

// This  class implements the masking model of Anbar & Anbar (1982).
import java.io.*;
import java.util.Vector;

public class AnbarAnbar extends Simulation implements Serializable
{        
    public AnbarAnbar()
    {
        Name = "Decaying trace model"; // title of GUI window
        Description = "This is a simulation of the decaying trace model described by Anbar & Anbar (1982). The starting parameters and stimulus characteristics are those used in Francis (2000).";

        Results = new SimResults(Name, "ISI (ms)", "Percept strength");  // initial labels of plot  
        
        // define Independent Variable parameters
        IVParameters = new Parameter[4];
        // set IV to be the ISI (parameter 6)
        IVParameters[0] = new Parameter("Independent variable", 6, "This indicates which parameter is to be used as the independent variable. Pick an integer that corresponds to one of the model parameters."); 
        IVParameters[1] = new Parameter("IV_Start", 0.0, "IV_Start is the starting value of the independent variable.");
        IVParameters[2] = new Parameter("IV_End", 200.0, "IV_End is the ending value of the independent variable.");
        IVParameters[3] = new Parameter("IV_Step", 10.0, "IV_Step is the size of steps through the independent variable."); 
        
        // define simulation parameters
        simParameters = new Parameter[8];
        simParameters[0] = new Parameter("Gamma", 0.3, "Gamma is an exponent that influences both the strength of the target signal and also the rate of decay of the target VRF. Anbar & Anbar (1982) set it equal to 0.3, based on models of brightness perception.");
        simParameters[1] = new Parameter("p", 0.442857143, "p is an exponent that influences the effect the mask has on the target. Larger values will tend to make the mask stronger.");
        simParameters[2] = new Parameter("G", 0.00000001, "G is a threshold. To compute percept strength the target VRF is integrated up to the time that the VRF goes below the value of G. Setting G smaller will tend to make the percept strength larger.");       
        simParameters[3] = new Parameter("Target intensity", 0.000001183, "Target intensity is the strength of the target input.");
        simParameters[4] = new Parameter("Target duration", 5.0, "Target duration is the duration of the target input.");
        simParameters[5] = new Parameter("Mask intensity", 0.000001183, "Mask intensity is the strength of the mask input.");
        simParameters[6] = new Parameter("ISI", 0.0, "ISI is the interstimulus interval between the target and the mask. This is the time between the offset of the target and the onset of the mask.");
        simParameters[7] = new Parameter("DeltaT", 0.001, "DeltaT is the temporal resolution of the simulation. Variables are checked once every DeltaT time steps. A smaller value will produce more accurate results, but will also cause the simulation to take longer."); 

        // define Variables
        Variables = new Parameter[2];
        Variables[0] = new Parameter("VRF", 0.0, "The VRF is a visual system's response to the target and mask stimuli. Percept strength is the integral of the VRF.");
        Variables[1] = new Parameter("Percept strength", 0.0, "The percept strength is a representation of the visiblity of the target. Larger values indicate better visibility.");
                                        
        // Save these as defaults
        IVParametersDefault = new Parameter[IVParameters.length];
        for(int i=0;i<IVParameters.length;i++)
            IVParametersDefault[i] = IVParameters[i].copy();
        simParametersDefault = new Parameter[simParameters.length];
        for(int i=0;i<simParameters.length;i++)
            simParametersDefault[i] = simParameters[i].copy();       
        variablesDefault = new Parameter[Variables.length];
        for(int i=0;i<Variables.length;i++)
            variablesDefault[i] = Variables[i].copy();   
    }
    
	public static void main(String args[]) 
	{
		AnbarAnbar simulation = new AnbarAnbar();
                new SimulationGUI(simulation);
	}    

        public boolean  UpdateParameters(Parameter[] newIVParameters, Parameter[] newSimParameters, Parameter[] newOutputParameters)
        {
            for(int i=0;i<IVParameters.length;i++)
                IVParameters[i].Value = newIVParameters[i].Value;           
            for(int i=0;i<simParameters.length;i++)
                simParameters[i].Value = newSimParameters[i].Value;     
                                                          
            return(true);
        }

	public boolean CheckParameters()
        {
            // not implemented
            return(true);
        }

    public void start()
    {
        done=false;
        animate = new Thread(this);
        animate.setPriority(5);
        animate.start();
    }

    public void run()
    {
        RunSimulation(this.CurrentRunName);
        done = true;
    }
    
    public void startRun(String conditionName)
    {
        this.CurrentRunName = conditionName;
        start();
    }
    
	public void RunSimulation(String conditionName)
        {
            Vector ResultsSequence = new Vector();
            int numberOfRuns = (int)((IVParameters[2].Value - IVParameters[1].Value)/IVParameters[3].Value);
            int numOfRunsSoFar=0;
            for(double IV = IVParameters[1].Value; IV <= IVParameters[2].Value; IV+= IVParameters[3].Value )
            {
                
                int percent = (int) ( 100*((float)numOfRunsSoFar/(float)numberOfRuns));
                numOfRunsSoFar++;
                Status = ""+percent;
                
                // set IV appropriately
                simParameters[(int)IVParameters[0].Value].Value = IV;
                double alpha = 0.0; // an intermediate term that represents the decay rate
                Variables[1].Value =0.0;  // set VRF to zero at start of a trial

                // now compute percept strength for this condition
                for(double time=0;time<20000;time+=simParameters[7].Value)
		{
                        // presence of target, no decay
			if(time>=0.0 && time<=simParameters[4].Value)
			{
				Variables[0].Value = simParameters[3].Value;
                                alpha=0.0;
			}
                        // offset of target causes a change in rate of decay
                        if(time >= simParameters[4].Value && time < simParameters[4].Value + simParameters[7].Value)
			{
                        	alpha = Math.pow(simParameters[3].Value, simParameters[0].Value);
                        }
                        // appearance of the mask
                        if(time>=simParameters[4].Value + simParameters[6].Value && time < simParameters[4].Value + simParameters[6].Value + simParameters[7].Value)
			{
                            Variables[0].Value = Variables[0].Value*Math.pow(Variables[0].Value/simParameters[5].Value,simParameters[1].Value);
                             alpha = Math.pow(Variables[0].Value,simParameters[0].Value);   
			}
                        // update VRF to incorporate decay
			Variables[0].Value += simParameters[7].Value*(-alpha*Variables[0].Value);

                        // update calculation of percept strength, integral of VRF above threshold
                        if(Variables[0].Value>simParameters[2].Value)
				Variables[1].Value += simParameters[7].Value*(Variables[0].Value-simParameters[2].Value); 
                        else if(time>simParameters[4].Value && Variables[0].Value<simParameters[2].Value)
                        // stop the simulation after target offset, once VRF goes below threshold
				time=20000;
		}
                // save data: Independent Variable and Percept Strength
                ResultPair result = new ResultPair(IV, Variables[1].Value);	              
                ResultsSequence.addElement(result);
            }
            SimLine Line = new SimLine(conditionName, ResultsSequence);
            Results.addLine(Line);
        }
}
