// Models of backward masking -- July 2002
// Greg Francis (gfrancis@psych.purdue.edu)
// Purdue University
// This material is based upon work supported by the National Science Foundation under Grant No. 0108905. 

// This  class implements the masking model of Bridgeman (1978).
import java.io.*;
import java.util.Vector;
import java.util.Random;

public class Bridgeman extends Simulation implements Serializable
{
    // Local variables that help make the code make sense for the model equations
    Random randGen = new Random();    
    public Bridgeman()
    {
        Name = "Recurrent inhibition model";
        Description = "This is a simulation of the recurrent inhibition model described by Bridgeman (1978). The starting parameters and stimulus characteristics are those used in Francis (2000).";

        
        Results = new SimResults(Name, "SOA (iteration)", "Percept strength");          
        // define Independent Variable parameters
        IVParameters = new Parameter[4];
        IVParameters[0] = new Parameter("Independent variable", 10, "This indicates which parameter is to be used as the independent variable."); 
        IVParameters[1] = new Parameter("IV_Start", 0.0, "IV_Start is the starting value of the independent variable.");
        IVParameters[2] = new Parameter("IV_End", 12.0, "IV_End is the ending value of the independent variable.");
        IVParameters[3] = new Parameter("IV_Step", 1.0, "IV_Step is the size of steps through the independent variable."); 
        
        // define simulation parameters
        simParameters = new Parameter[15];
        simParameters[0] = new Parameter("numCells", 30, "numCells is the number of cells in the network.");
        simParameters[1] = new Parameter("Target intensity", 22.5, "Target intensity is the luminance of the target input.");
        simParameters[2] = new Parameter("Target duration", 2.0, "Target duration is the duration of the target input. It should be an integer.");       
        simParameters[3] = new Parameter("Mask intensity", 22.5, "Mask intensity is the strength of the mask input.");
        simParameters[4] = new Parameter("Mask duration", 2.0, "Mask duration is the duration of the mask input. It should be an integer.");
        simParameters[5] = new Parameter("Target size", 2, "Target size is the number of contiguous cells that are stimulated by the target input.");
        simParameters[6] = new Parameter("Mask size", 1, "Mask size is the number of contiguous cells that are stimulated by the mask input. The mask flanks the target on each side, so the total number of cells stimulated by the mask will be twice this number.");
        simParameters[7] = new Parameter("Background", 50, "Background is the luminance of the background.  The target and mask inputs are added to this Background value."); 
        simParameters[8] = new Parameter("Separation", 1, "Separation is the number of cells located between the cells that are stimulated by the target and mask, respectively.");         
        simParameters[9] = new Parameter("NoiseScale", 0.1, "NoiseScale multiplies the value of an additive noise component."); 
        simParameters[10] = new Parameter("SOA", 0, "SOA is the stimulus onset asynchrony between the target and the mask. It is the time between the onset of the target and the onset of the mask. It should be an integer.");  
        simParameters[11] = new Parameter("w1", 0.3, "w1 is the strength of the inhibitory connection between two cells that are next to each other. The signal is sent with a delay of one time step.");  
        simParameters[12] = new Parameter("w2", 0.3, "w2 is the strength of the inhibitory connection between two cells that are two pixels apart. The signal is sent with a delay of two time steps.");  
        simParameters[13] = new Parameter("w3", 0.1, "w3 is the strength of the inhibitory connection between two cells that are three pixels apart. The signal is sent with a delay of three time steps.");          
        simParameters[14] = new Parameter("numIterations", 12, "numIterations is the number of time steps that the simulation runs after onset of the target.");          
                
        // define Variables
        Variables = new Parameter[2];
        Variables[0] = new Parameter("VRF", 0.0, "The VRF is a visual system's response to the target and mask stimuli. Percept strength is the integral of the VRF. In Bridgeman's model this is the squared correlation between a condition with a mask and a condition without a mask.");
        Variables[1] = new Parameter("Percept strength", 0.0, "The percept strength is a representation of the visiblity of the target. Larger values indicate better visibility.");
                                        
        // Save these as defaults
        IVParametersDefault = new Parameter[IVParameters.length];
        for(int i=0;i<IVParameters.length;i++)
            IVParametersDefault[i] = IVParameters[i].copy();
        simParametersDefault = new Parameter[simParameters.length];
        for(int i=0;i<simParameters.length;i++)
            simParametersDefault[i] = simParameters[i].copy();       
        variablesDefault = new Parameter[Variables.length];
        for(int i=0;i<Variables.length;i++)
            variablesDefault[i] = Variables[i].copy();   
    }
    
	public static void main(String args[]) 
	{
		Bridgeman simulation = new Bridgeman();
                new SimulationGUI(simulation);
	}    

        public boolean  UpdateParameters(Parameter[] newIVParameters, Parameter[] newSimParameters, Parameter[] newOutputParameters)
        {
            for(int i=0;i<IVParameters.length;i++)
                IVParameters[i].Value = newIVParameters[i].Value;           
            for(int i=0;i<simParameters.length;i++)
                simParameters[i].Value = newSimParameters[i].Value;     
                                                          
            return(true);
        }

	public boolean CheckParameters()
        {
            // not implemented
            return(true);
        }

    public void start()
    {
        done=false;
        animate = new Thread(this);
        animate.setPriority(5);
        animate.start();
    }

    public void run()
    {
        RunSimulation(this.CurrentRunName);
        done = true;
    }
    
    public void startRun(String conditionName)
    {
        this.CurrentRunName = conditionName;
        start();
    }
    
	public void RunSimulation(String conditionName)
        {
            Vector ResultsSequence = new Vector();
            int numberOfRuns = (int)((IVParameters[2].Value - IVParameters[1].Value)/IVParameters[3].Value);
            int numOfRunsSoFar=0;
            for(double IV = IVParameters[1].Value; IV <= IVParameters[2].Value; IV+= IVParameters[3].Value )
            {
                int percent = (int) ( 100*((float)numOfRunsSoFar/(float)numberOfRuns));
                numOfRunsSoFar++;
                Status = ""+percent;
                
                // set IV appropriately
                simParameters[(int)IVParameters[0].Value].Value = IV;
                double alpha = 0.0; // an intermediate term that represents the decay rate
                Variables[1].Value =0.0;
                double x2 = 0.0;
                
                double x[][] = new double[(int)simParameters[0].Value][(int)simParameters[14].Value] ; // [position][time] // mask condition
                double y[][] = new double[(int)simParameters[0].Value][(int)simParameters[14].Value]; // [position][time] // no mask condition              
                
                // Initialize the equations for the background luminance
                for(int h=0;h<200/simParameters[14].Value;h++) 
		{
                    initializeEquations(x);
                    initializeEquations(y);
		}     
                
                // run simulation without mask
		runEquations(x, 0.0);
                                    
                // run equations with the mask
                runEquations(y, simParameters[3].Value);                                        
                
                // compute the correlation between the conditions
                Variables[1].Value = computeCorrelation(x,y);
                
                // save data: Indepenent Variable and Percept Strength
                ResultPair result = new ResultPair(IV, Variables[1].Value);	              
                ResultsSequence.addElement(result);
            }
            SimLine Line = new SimLine(conditionName, ResultsSequence);
            Results.addLine(Line);
        }
        
	public void initializeEquations(double[][] x)
	{
            int numCells = (int)simParameters[0].Value;
            for(int i=3;i<simParameters[14].Value;i++)
            {
                for(int k=0;k<numCells;k++)
                {
                    double inhibition=0.0;
                    // go over nearby cells
                    for(int j=-3;j<=3;j++)
                    {
                        int temp = k+j;
                        if(temp<0)
                            temp= numCells-temp;
                            temp = temp%numCells;
                            switch(Math.abs(j))
                            {
                                case 1:
                                inhibition += simParameters[11].Value* (x[temp][i-Math.abs(j)]);
                                break;
						
                                case 2:
                                inhibition += simParameters[12].Value* (x[temp][i-Math.abs(j)]);
                                break;
						
                                case 3:
                                inhibition += simParameters[13].Value* (x[temp][i-Math.abs(j)]);
                                break;
                            }
                    }
                    double excitation=simParameters[7].Value;
                                                                    
                    x[k][i] = Math.max(excitation-inhibition+simParameters[9].Value*GaussianNoise(),0.0);
                }	
        }
        //move last three data values to first three data values
        for(int i=0;i<3;i++)
        for(int k=0;k<numCells;k++)
            x[k][i] = x[k][(int)simParameters[14].Value-3+i];
	
    }
    
	public void runEquations(double[][] x, double MaskLum)
	{
            int numCells = (int)simParameters[0].Value;
            for(int i=3;i<(int)simParameters[14].Value;i++)
            {
                for(int k=0;k<numCells;k++)
                {
                    double inhibition=0.0;
                    // go over nearby cells to compute inhibition
                    for(int j=-3;j<=3;j++)
                    {
                        int temp = k+j;
                        if(temp<0)
                            temp= numCells-temp;
                            temp = temp%numCells;
                            switch(Math.abs(j))
                            {
                                case 1:
                                inhibition += simParameters[11].Value* (x[temp][i-Math.abs(j)]);
                                break;
						
                                case 2:
                                inhibition += simParameters[12].Value* (x[temp][i-Math.abs(j)]);
                                break;
						
                                case 3:
                                inhibition += simParameters[13].Value* (x[temp][i-Math.abs(j)]);
                                break;
                            }
                    }
                    double excitation=simParameters[7].Value;
                                
                    // center target input      
                    // target appears at time 3      
                    if(i>=3 && i<3+(int)simParameters[2].Value && Math.abs(numCells/2 +0.5 - k)<=(int)simParameters[5].Value)
                    {
                        excitation+=simParameters[1].Value;
                    }
                    // mask around the target
                    int Separation=(int)simParameters[5].Value+(int)simParameters[6].Value+(int)simParameters[8].Value;
                    if(i>=3+(int)simParameters[10].Value && i<3+(int)simParameters[10].Value + (int)simParameters[4].Value && (Math.abs(numCells/2+0.5-Separation-k) <=simParameters[6].Value || Math.abs(numCells/2+0.5+Separation-k)<=simParameters[6].Value))
                    {
                        excitation+=MaskLum;
                    }
					
                    x[k][i] = Math.max(excitation-inhibition+simParameters[9].Value*GaussianNoise(),0.0);
									
                }	
            }
	
	}

	public double computeCorrelation(double[][] x, double [][] y)
	{
            int numCells = (int)simParameters[0].Value;
            double visability=0.0;
            for(int i=3;i<(int)simParameters[14].Value;i++)
            {
                double temp1=0.0;
                double temp2=0.0;
                double temp3=0.0;
                double temp4=0.0;
                double temp5=0.0;
                for(int k=0;k<numCells;k++)
                {
                    temp1+=x[k][i];
                    temp2+=y[k][i];
                    temp3+= Math.pow(x[k][i],2.0);
                    temp4+= Math.pow(y[k][i],2.0);
                    temp5+= x[k][i]*y[k][i];
                }
			
                Variables[0].Value = (numCells*temp5 - temp1*temp2)/Math.sqrt((numCells*temp3-Math.pow(temp1,2.0))*(numCells*temp4-Math.pow(temp2,2.0)));
			
                visability += Math.pow(Variables[0].Value,2.0)/((int)simParameters[14].Value-3);
            }
          
            return(visability);
	}	
    
	public double GaussianNoise()
	{
		double z1=1;
		double S=2.0;
		while(S>1.0)
		{
			double v1= 2.0*randGen.nextDouble()-1.0;
			double v2 = 2.0*randGen.nextDouble()-1.0;
			S= Math.pow(v1,2.0)+ Math.pow(v2,2.0);
			z1 = v1*Math.sqrt(-2.0*Math.log(S)/S); 
		}
		return(z1);
	}
}
