// Models of backward masking -- July 2002
// Greg Francis (gfrancis@psych.purdue.edu)
// Purdue University
// This material is based upon work supported by the National Science Foundation under Grant No. 0108905. 

// This  class implements the masking model of Di Lollo, Enns and Rensink (2000).
import java.io.*;
import java.util.Vector;

public class DiLolloEtAl extends Simulation implements Serializable
{        
    public DiLolloEtAl()
    {
        Name = "Reentrant processing model";
        Description = "This is a simulation of the reentrant processing model described by Di Lollo, Enns & Rensink (2000). The starting parameters and stimulus characteristics are those to create Figure 15 A in Di Lollo et al. (2000).";
        
        Results = new SimResults(Name, "Mask duration", "Percept strength"); 
        
        // define Independent Variable parameters
        IVParameters = new Parameter[4];
        IVParameters[0] = new Parameter("Independent variable", 5, "This indicates which parameter is to be used as the independent variable. Pick an integer that corresponds to one of the model parameters."); 
        IVParameters[1] = new Parameter("IV_Start", 10.0, "IV_Start is the starting value of the independent variable.");
        IVParameters[2] = new Parameter("IV_End", 330, "IV_End is the ending value of the independent variable.");
        IVParameters[3] = new Parameter("IV_Step", 45.0, "IV_Step is the size of steps through the independent variable."); 
        
        // define simulation parameters
        simParameters = new Parameter[12];
        simParameters[0] = new Parameter("Target intensity", 1.0, "Target intensity is the strength of the target input.");
        simParameters[1] = new Parameter("Mask intensity", 1.0, "Mask intensity is the strength of the mask input.");
        simParameters[2] = new Parameter("Noise intensity", 0.4, "Noise intensity is the strength of the noise signal.");       
        simParameters[3] = new Parameter("Lambda", 0.025, "Lambda scales the stimulus input to the model. The feedback scale factor is always 1.0.");
        simParameters[4] = new Parameter("Target duration", 10.0, "Target duration is the duration of the target input in milliseconds.");
        simParameters[5] = new Parameter("Mask duration", 10.0, "Mask duration is the duration of the mask input in milliseconds. This differs from what Di Lollo, Enns & Rensink (2000) called mask duration. They used the term to indicate the duration of time that the mask stimulus was presented after the target turned off.");
        simParameters[6] = new Parameter("SOA", 0.0, "SOA is the stimulus onset asynchrony between the target and the mask. This is the time between the onset of the target and the onset of the mask, in milliseconds.");
        simParameters[7] = new Parameter("n", 4, "n is the set size, indicating the number of elements that are presented with the target."); 
        simParameters[8] = new Parameter("S", 12, "S is the search rate by which attention moves from item to item.  Di Lollo et al. (2000) published parameters in Table 1."); 
        simParameters[9] = new Parameter("K", 0.525, "K sets an asympotic value for which probability correct approaches the value 1.0.");      
        simParameters[10] = new Parameter("Time step", 13, "Time step is the number of milliseconds that correspond to each reentrant iteration.");     
        simParameters[11] = new Parameter("Guess rate", 0.25, "Guess rate links the value of the model to accuracy of target detection. The guess rate is what would correspond to the hit rate if subjects just guessed in the experiment.");  
    
        // define Variables
        Variables = new Parameter[4];
        Variables[0] = new Parameter("P1", 0.0, "The representation of the target.");
        Variables[1] = new Parameter("P2", 0.0, "The representation of the target.");
        Variables[2] = new Parameter("P3", 0.0, "The representation of the noise signal.");
        Variables[3] = new Parameter("Probablity correct", 0.0, "The model's behavior on a trial.");
 
        // Save these as defaults
        IVParametersDefault = new Parameter[IVParameters.length];
        for(int i=0;i<IVParameters.length;i++)
            IVParametersDefault[i] = IVParameters[i].copy();
        simParametersDefault = new Parameter[simParameters.length];
        for(int i=0;i<simParameters.length;i++)
            simParametersDefault[i] = simParameters[i].copy();       
        variablesDefault = new Parameter[Variables.length];
        for(int i=0;i<Variables.length;i++)
            variablesDefault[i] = Variables[i].copy();   
    }
    
	public static void main(String args[]) 
	{
		DiLolloEtAl simulation = new DiLolloEtAl();
                new SimulationGUI(simulation);
	}    

        public boolean  UpdateParameters(Parameter[] newIVParameters, Parameter[] newSimParameters, Parameter[] newOutputParameters)
        {
            for(int i=0;i<IVParameters.length;i++)
                IVParameters[i].Value = newIVParameters[i].Value;           
            for(int i=0;i<simParameters.length;i++)
                simParameters[i].Value = newSimParameters[i].Value;     
            return(true);
        }

	public boolean CheckParameters()
        { 
            // not implemented
            return(true);
        }

    public void start()
    {
        done=false;
        animate = new Thread(this);
        animate.setPriority(5);
        animate.start();
    }

    public void run()
    {
        RunSimulation(this.CurrentRunName);
        done = true;
    }
    
    public void startRun(String conditionName)
    {
        this.CurrentRunName = conditionName;
        start();
    }
    
	public void RunSimulation(String conditionName)
        {
            Vector ResultsSequence = new Vector();
            int numberOfRuns = (int)((IVParameters[2].Value - IVParameters[1].Value)/IVParameters[3].Value);
            int numOfRunsSoFar=0;
            for(double IV = IVParameters[1].Value; IV <= IVParameters[2].Value; IV+= IVParameters[3].Value )
            {
                int percent = (int) ( 100*((float)numOfRunsSoFar/(float)numberOfRuns));
                numOfRunsSoFar++;
                Status = ""+percent;                // set IV appropriately
                simParameters[(int)IVParameters[0].Value].Value = IV;
                
                 // initialize activities to zero
                for(int i=0;i<Variables.length;i++)
                    Variables[i].Value = 0.0;
                
                // compute the time to focus attention and to stop the simulation 
                int TimeToStop = (int)(simParameters[7].Value * simParameters[8].Value);
                double TargetLum=0.0, MaskLum=0.0;
                int startTime=(int)Math.min(0, simParameters[6].Value);
                
                for(int time = startTime; time <=TimeToStop;time+=simParameters[10].Value)
                {
                    // set target intensities for this time step
                    if(time>=0 && time < (int)simParameters[4].Value)
                    {
                        TargetLum = simParameters[0].Value;
                    }    
                    else
                        TargetLum = 0.0;
                
                    // set mask intensities for this time step
                    if(time>= (int)simParameters[6].Value && time < (int)simParameters[5].Value + (int)simParameters[6].Value)
                        MaskLum = simParameters[1].Value;
                    else
                        MaskLum = 0.0;                
                                                    
                    // compute total activity of past P values
                    double sum=0.0;
                    for(int i=0;i<3;i++)
                        sum+= Math.pow(Variables[i].Value,2.0);
                    sum = Math.sqrt(sum);
                    
                    // catch division by zero and replace it with division by 1.0
                    if(sum==0.0)
                        sum=1.0;
                                        
                    // update P values for next time step
                    //target
                    Variables[0].Value = Variables[0].Value/sum + simParameters[3].Value * TargetLum;
                    //mask
                    Variables[1].Value = Variables[1].Value/sum + simParameters[3].Value * MaskLum;
                    // noise
                    Variables[2].Value = Variables[2].Value/sum + simParameters[3].Value * simParameters[2].Value;
                }
                
                // simulation ended, now compute VRF
   
                // compute total activity
                double sum=0.0;
                for(int i=0;i<3;i++)
                    sum+= Math.pow(Variables[i].Value,2.0);
                                     
                Variables[3].Value = Math.min( Math.pow(Variables[0].Value,2.0)/(sum*simParameters[9].Value), 1.0);
                
                // correct the output for guessing behavior
                double correctedHit = 1.0 - (1.0 -Variables[3].Value)*(1.0-simParameters[11].Value);
                
                // save data: Independent Variable and Percept Strength
                ResultPair result = new ResultPair(IV, correctedHit); 	              
                ResultsSequence.addElement(result);
            }
            SimLine Line = new SimLine(conditionName, ResultsSequence);
            Results.addLine(Line);
        }
}
