// Models of backward masking -- July 2002
// Greg Francis (gfrancis@psych.purdue.edu)
// Purdue University
// This material is based upon work supported by the National Science Foundation under Grant No. 0108905. 

// This  class implements the efficient masking model of Francis (2000).
import java.io.*;
import java.util.Vector;

public class EfficientMasking extends Simulation implements Serializable
{        
    public EfficientMasking()
    {
        Name = "Efficient masking model";
        Description = "This is a simulation of the efficient masking model first described by Francis (2000). The starting parameters and stimulus characteristics are those used in Francis (in press, Cognitive Psychology).";
        
        Results = new SimResults(Name, "SOA (ms)", "Percept strength");  
                
        // define Independent Variable parameters
        IVParameters = new Parameter[4];
        IVParameters[0] = new Parameter("Independent variable", 10, "This indicates which parameter is to be used as the independent variable. Pick an integer that corresponds to one of the model parameters."); 
        IVParameters[1] = new Parameter("IV_Start", 0.0, "IV_Start is the starting value of the independent variable.");
        IVParameters[2] = new Parameter("IV_End", 200.0, "IV_End is the ending value of the independent variable.");
        IVParameters[3] = new Parameter("IV_Step", 10.0, "IV_Step is the size of steps through the independent variable."); 
        
        // define simulation parameters
        simParameters = new Parameter[12];
        simParameters[0] = new Parameter("A", 0.01, "A sets the rate of decay of the target VRF.");
        simParameters[1] = new Parameter("B", 1.0, "B scales the additive contribtion of the target input. When C >0 it also provides an upper limit on the magnitude of the target VRF.");
        simParameters[2] = new Parameter("C", 1.0, "C scales the contribution of the target input to the rate of change of the target VRF.");       
        simParameters[3] = new Parameter("D", 0.00031, "D scales the additive contribution of the mask input.");
        simParameters[4] = new Parameter("E", 0.001, "E scales the contribution of the mask input to the rate of change of the target VRF.");
        simParameters[5] = new Parameter("G", 0.2, "G is a threshold. To compute percept strength the target VRF is computed up to the time that the VRF goes below the value of G. Setting G smaller will tend to make the percept strength larger.");
        simParameters[6] = new Parameter("Target intensity", 10, "Target intensity is the strength of the target input."); 
        simParameters[7] = new Parameter("Target duration", 15, "Target duration is the duration of the target input."); 
        simParameters[8] = new Parameter("Mask intensity", 10, "Mask intensity is the strength of the mask input."); 
        simParameters[9] = new Parameter("Mask duration", 15, "Mask duration is the duration of the mask input."); 
        simParameters[10] = new Parameter("SOA", 0.0, "SOA is the stimulus onset asynchrony between the target and the mask. This is the time between the onset of the target and the onset of the mask.");
        simParameters[11] = new Parameter("DeltaT", 0.001, "DeltaT is the temporal resolution of the simulation. Variables are checked once every DeltaT time steps. Smaller values will cause the simulation to take longer. Smaller values also produce more accurate calculations of the differential equations.");
        
        // define Variables
        Variables = new Parameter[2];
        Variables[0] = new Parameter("VRF", 0.0, "The VRF is a visual system's response to the target and mask stimuli. Percept strength is the integral of the VRF.");
        Variables[1] = new Parameter("Percept strength", 0.0, "The percept strength is a representation of the visiblity of the target. Larger values indicate better visibility.");
                                        
        // Save these as defaults
        IVParametersDefault = new Parameter[IVParameters.length];
        for(int i=0;i<IVParameters.length;i++)
            IVParametersDefault[i] = IVParameters[i].copy();
        simParametersDefault = new Parameter[simParameters.length];
        for(int i=0;i<simParameters.length;i++)
            simParametersDefault[i] = simParameters[i].copy();       
        variablesDefault = new Parameter[Variables.length];
        for(int i=0;i<Variables.length;i++)
            variablesDefault[i] = Variables[i].copy();   
    }
    
	public static void main(String args[]) 
	{
		EfficientMasking simulation = new EfficientMasking();
                new SimulationGUI(simulation);
	}    

        public boolean  UpdateParameters(Parameter[] newIVParameters, Parameter[] newSimParameters, Parameter[] newOutputParameters)
        {
            for(int i=0;i<IVParameters.length;i++)
                IVParameters[i].Value = newIVParameters[i].Value;           
            for(int i=0;i<simParameters.length;i++)
                simParameters[i].Value = newSimParameters[i].Value;     
                                                          
            return(true);
        }

	public boolean CheckParameters()
        {
            // not implemented
            return(true);
        }

    public void start()
    {
        done=false;
        animate = new Thread(this);
        animate.setPriority(5);
        animate.start();
    }

    public void run()
    {
        RunSimulation(this.CurrentRunName);
        done = true;
    }
    
    public void startRun(String conditionName)
    {
        this.CurrentRunName = conditionName;
        start();
    }
    
	public void RunSimulation(String conditionName)
        {
            Vector ResultsSequence = new Vector();
                                
            int numberOfRuns = (int)((IVParameters[2].Value - IVParameters[1].Value)/IVParameters[3].Value);
            int numOfRunsSoFar=0;
            for(double IV = IVParameters[1].Value; IV <= IVParameters[2].Value; IV+= IVParameters[3].Value )
            {
                int percent = (int) ( 100*((float)numOfRunsSoFar/(float)numberOfRuns));
                numOfRunsSoFar++;
                Status = ""+percent;
                // set IV appropriately
                simParameters[(int)IVParameters[0].Value].Value = IV;

                // initialize cell activities to zero
                for(int i=0;i<Variables.length;i++)
                    Variables[i].Value = 0.0;
                    
                // now compute percept strength for this condition
                double TargetLuminance = 0.0, MaskLuminance = 0.0;
                
                // find start time, needs to be negative value if SOA is negative
                double time = 0.0;
                if(simParameters[10].Value < 0)
                    time = simParameters[10].Value;
                boolean beenAboveThreshold=false;
                boolean stopSimulation=false;
                double OnTime=0;
                while(!stopSimulation)
                {
                    // Set up intensities of target for this time step
                    if(time>=0.0 && time<simParameters[7].Value)
                        TargetLuminance = simParameters[6].Value;
                    else
                        TargetLuminance=0.0;
  
                    // Set up intensities of mask for this time step
                    if(time>=simParameters[10].Value && time < simParameters[10].Value + simParameters[9].Value)
                        MaskLuminance=simParameters[8].Value;
                    else
                        MaskLuminance=0.0;   

                    // Update target VRF
                    Variables[0].Value += simParameters[11].Value*(-simParameters[0].Value * Variables[0].Value + (simParameters[1].Value - simParameters[2].Value* Variables[0].Value) * TargetLuminance - (simParameters[3].Value + simParameters[4].Value* Variables[0].Value) * MaskLuminance );
                                        
                    // note OnTime, when VRF goes above threshold
                    if(!beenAboveThreshold)
                    {
                        if(Variables[0].Value > simParameters[5].Value)
                        {
                            OnTime=time;
                            beenAboveThreshold=true;
                        }
                    }
                    
                    // If VRF never goes above threshold, percept strength is zero
                    if(time>simParameters[7].Value && !beenAboveThreshold)
                    {
                        stopSimulation=true;
                        Variables[1].Value=0;
                    } 
                    
                    // note stop time, when VRF goes below threshold
                    if(beenAboveThreshold && Variables[0].Value < simParameters[5].Value)
                    {
                        stopSimulation=true;
                        Variables[1].Value = time-OnTime;  // percept time is the duration the VRF is above threshold
                    }
                    // update time
                    time+= simParameters[11].Value;
                }
                
                // save data: Independent Variable and Percept Strength
                ResultPair result = new ResultPair(IV, Variables[1].Value);	              
                ResultsSequence.addElement(result);
            }
            SimLine Line = new SimLine(conditionName, ResultsSequence);
            Results.addLine(Line);
        }
}
