// Models of backward masking -- July 2002
// Greg Francis (gfrancis@psych.purdue.edu)
// Purdue University
// This material is based upon work supported by the National Science Foundation under Grant No. 0108905.

// This  class replicates (and corrects) the equations in Bachmann (1994).
// June 2004
// Supported by a NATO Collaborative Linkage grant to Gregory Francis (Michael Herzog, PI).
import java.io.*;
import java.util.Vector;

public class PerceptualRetouch extends Simulation implements Serializable
{
    public PerceptualRetouch()
    {
        Name = "Perceptual Retouch model";
        Description = "This is a simulation of the perceptual retouch quantitative model proposed by Bachmann (1994). It follows the equations in that description, but also simplifies them a bit and corrects a few mistakes.";

        Results = new SimResults(Name, "SOA (ms)", "Percept strength");

        // define Independent Variable parameters
        IVParameters = new Parameter[4];
        IVParameters[0] = new Parameter("Independent variable", 7, "This indicates which parameter is to be used as the independent variable. Pick an integer that corresponds to one of the model parameters.");
        IVParameters[1] = new Parameter("IV_Start", 0.0, "IV_Start is the starting value of the independent variable.");
        IVParameters[2] = new Parameter("IV_End", 250.0, "IV_End is the ending value of the independent variable.");
        IVParameters[3] = new Parameter("IV_Step", 10.0, "IV_Step is the size of steps through the independent variable.");

        // define simulation parameters
        simParameters = new Parameter[19];
        simParameters[0] = new Parameter("a", 0.8, "a sets the peak of the initial EPSP reponse of the visual system to a stimuulus. Following the convention set in Bachmann (1994), one time unit corresponds to 33.33 milliseconds.");
        simParameters[1] = new Parameter("PR latency", 110.0, "PR (Perceptual retouch) latency is the delay (in milliseconds) between the onset of a stimulus and the appearance of the non-specific signal generated by that stimulus from lateral geniculate to cortex. \n\nFor SOAs greater than 'Min PR SOA', the EPSP for S2 is calculated at the time 'PR latency' - 'Min PR SOA', so the model will produce nonsense results if 'Min PR SOA' > 'PR latency'. ");
        simParameters[2] = new Parameter("S1 intensity", 10, "S1 intensity is the strength of the first stimulus. Because a logarithm is taken, this should never be less than 1.0. However, if it is set equal to zero, the logarithm will not be used, and it will be as if S1 is not presented at all.");
        simParameters[3] = new Parameter("k1", 8, "k1 is a constant that scales the magnitude of the non-specific response.");
        simParameters[4] = new Parameter("S2 intensity", 10, "S2 intensity is the strength of the second stimulus. Because a logarithm is taken, this should never be less than 1.0. However, if it is set equal to zero, the logarithm will not be used, and it will be as if S2 is not presented at all.");
        simParameters[5] = new Parameter("Min PR SOA", 80, "Min PR SOA helps to describe the dynamic response of the non-specific response to S1 as it influences the response to S2. SOAs smaller than Min PR SOA have a maximum PR effect. When the SOA is bigger than Min PR SOA, it is assumed that the non-specific repsonse has faded linearly, finally reaching zero at an SOA of 'Max PR SOA'. \n\nFor SOAs greater than 'Min PR SOA', the EPSP for S2 is calculated at the time 'PR latency' - 'Min PR SOA', so the model will produce nonsensible results if 'Min PR SOA' > 'PR latency'. ");
        simParameters[6] = new Parameter("Max PR SOA", 330, "Max PR SOA helps to describe the dynamic response of the non-specific response. Max PR SOA describes an upper limit on the SOA where the non-specific response from S1 influences the specific response from S2. When the SOA is bigger than 'Min PR SOA', it is assumed that the non-specific repsonse has faded linearly, finally reaching zero at an SOA of Max PR SOA. ");
        simParameters[7] = new Parameter("SOA (ms)", 0.0, "SOA is the stimulus onset asynchrony between the target and the mask. This is the time between the onset of the target and the onset of the mask. Because of the details of the simulation it never makes sense to use a negative SOA.");
        simParameters[8] = new Parameter("P[gn]", 0.2, "A guessing rate, as might be part of some experiments or might be due to spontaneous activity in the nervous system. Set P[gn]=0 to have guessing not involved at all.");
        simParameters[9] = new Parameter("Attention min SOA", 50, "The minimum SOA before attention effects that produce a bias for S2 begin. Note, this term and 'Attention max SOA' are used to compute k2, a model parameter in Bachmann (1994).");
        simParameters[10] = new Parameter("Attention max SOA", 150, "The maximum SOA for which attention effects that produce a bias for S2 are not maximum. In Bachmann (1994) this was called parameter k3. Note, this term and 'Attention min SOA' are used to compute k2, a model parameter in Bachmann (1994).");
        simParameters[11] = new Parameter("k4", 0.18, "The maximum attentional bias for S2.");
        simParameters[12] = new Parameter("k5", 0.139, "A final weighting term for the attentional bias toward S2. Set k5=0 to remove attention effects from the model. It has no effect on the percept strength of S1.");
        simParameters[13] = new Parameter("Cycle min SOA", 80, "The minimum SOA for which it is possible for more than one retouch cycle to occur. If SOA is less than this value, all computations are treated as if there is only one cycle to consider. That is, S1 and S2 are being treated as a single event.");
        simParameters[14] = new Parameter("Cycle max SOA", 250, "The maximum SOA for which it is possible for more than one retouch cycle to occur. If SOA is more than this value, all computations are treated as if S1 and S2 are in different cycles, and performance will be 100% correct.");
        simParameters[15] = new Parameter("Cycle weight", 1.0, "An extra parameter useful for judging the effect of having cycle effects present or absent. Set this equal to zero to have no cycle effects for any SOA. This would be treating S1 and S2 as a unitary percept, regardless of SOA.");
        simParameters[16] = new Parameter("Percept choice", 0.0, "The model produces a percept strength for both S1 and S2. You can choose which stimulus percept strength is plotted. Choose between S1 (0) or S2 (1).");
        simParameters[17] = new Parameter("Time scale", 0.03, "The relationship between model time and millisecond time. This value in model time corresponds to one millisecond in real time.");
        simParameters[18] = new Parameter("Use retouch", 1.0, "This parameter, if set equal to zero, removes the calculations involving perceptual retouch. It is useful for exploring the behavior of the model.");

        // define Variables
        Variables = new Parameter[8];
        Variables[0] = new Parameter("S1 specific response", 0.0, "The S1 response is a visual system's initial response to the first stimulus. ");
        Variables[1] = new Parameter("S2 specific response", 0.0, "The S2 response is a visual system's initial response to the second stimulus.");
        Variables[2] = new Parameter("Non-specific response", 0.0, "This is the non-specific response to either of the stimuli. ");
		Variables[3] = new Parameter("S1 PR", 0.0, "The response to the first stimulus after perceptual retouch. ");
		Variables[4] = new Parameter("S2 PR", 0.0, "The response to the second stimulus after perceptual retouch. ");
		Variables[5] = new Parameter("S1 Command", 0.0, "The command neuron response to the first stimulus.");
		Variables[6] = new Parameter("S2 Command", 0.0, "The command neuron response to the second stimulus.");
		Variables[7] = new Parameter("Percept", 0.0, "Percept of whatever is requested. ");

        // Save these as defaults
        IVParametersDefault = new Parameter[IVParameters.length];
        for(int i=0;i<IVParameters.length;i++)
            IVParametersDefault[i] = IVParameters[i].copy();
        simParametersDefault = new Parameter[simParameters.length];
        for(int i=0;i<simParameters.length;i++)
            simParametersDefault[i] = simParameters[i].copy();
        variablesDefault = new Parameter[Variables.length];
        for(int i=0;i<Variables.length;i++)
            variablesDefault[i] = Variables[i].copy();
    }

	public static void main(String args[])
	{
		PerceptualRetouch simulation = new PerceptualRetouch();
                new SimulationGUI(simulation);
	}

        public boolean  UpdateParameters(Parameter[] newIVParameters, Parameter[] newSimParameters, Parameter[] newOutputParameters)
        {
            for(int i=0;i<IVParameters.length;i++)
                IVParameters[i].Value = newIVParameters[i].Value;
            for(int i=0;i<simParameters.length;i++)
                simParameters[i].Value = newSimParameters[i].Value;

            return(true);
        }

	public boolean CheckParameters()
        {
            // not implemented
            return(true);
        }

    public void start()
    {
        done=false;
        animate = new Thread(this);
        animate.setPriority(5);
        animate.start();
    }

    public void run()
    {
        RunSimulation(this.CurrentRunName);
        done = true;
    }

    public void startRun(String conditionName)
    {
        this.CurrentRunName = conditionName;
        start();
    }

	public void RunSimulation(String conditionName)
        {
            Vector ResultsSequence = new Vector();

            int numberOfRuns = (int)((IVParameters[2].Value - IVParameters[1].Value)/IVParameters[3].Value);
            int numOfRunsSoFar=0;
            for(double IV = IVParameters[1].Value; IV <= IVParameters[2].Value; IV+= IVParameters[3].Value )
            {
                int percent = (int) ( 100*((float)numOfRunsSoFar/(float)numberOfRuns));
                numOfRunsSoFar++;
                Status = ""+percent;
                // set IV appropriately
                simParameters[(int)IVParameters[0].Value].Value = IV;

                // initialize cell activities to zero
                for(int i=0;i<Variables.length;i++)
                    Variables[i].Value = 0.0;

                // now compute percept strength for this condition
                double S1Luminance = simParameters[2].Value, S2Luminance = simParameters[4].Value;

				// avoid negative intensities after the log
				if(simParameters[2].Value >= 1.0)
					S1Luminance = Math.log(simParameters[2].Value);
				else
					S1Luminance = 0.0;
				if(simParameters[4].Value >= 1.0)
					S2Luminance = Math.log(simParameters[4].Value);
				else
					S2Luminance = 0.0;

                    // Update PR1 (equation 7)
                    double PRlatency = simParameters[17].Value * simParameters[1].Value;
                    double SOA=simParameters[7].Value;
                    Variables[3].Value = EPSP(S1Luminance, simParameters[0].Value, PRlatency);

					// add in retouch
					if(simParameters[18].Value !=0.0)
					{
						Variables[3].Value *=  S1Luminance * simParameters[3].Value;
					}

                    // Update PR2
                    if(SOA <= simParameters[5].Value)
                    {
						Variables[4].Value = EPSP(S2Luminance, simParameters[0].Value, simParameters[17].Value*Math.max(simParameters[1].Value-SOA,0.0));
						// add in retouch
						if(simParameters[18].Value !=0.0)
						{
							Variables[4].Value *=  S1Luminance * simParameters[3].Value;
						}
					}
					else
					{
						Variables[4].Value = EPSP(S2Luminance, simParameters[0].Value, simParameters[17].Value*Math.max(simParameters[1].Value-simParameters[5].Value,0.0));
						// add in retouch
						if(simParameters[18].Value !=0.0)
						{
							Variables[4].Value *=   S1Luminance * simParameters[3].Value * Math.max(simParameters[6].Value-SOA,0.0)/simParameters[6].Value; // originally divided by 250, but this seems to make more sense

						}
					}

					double sum = Variables[3].Value + Variables[4].Value;
					if(sum==0.0)
						sum=1.0;
					// Update S1 command
                    Variables[5].Value = Variables[3].Value/sum;
					// Update S2 command
                    Variables[6].Value = Variables[4].Value/sum;

					// Take guessing into account
					double g1 =  simParameters[8].Value *(1-Variables[5].Value);
					double g2 =  simParameters[8].Value *(1-Variables[6].Value);

					// Take attentional effects into account for S2
					double k2 = simParameters[11].Value/Math.pow(simParameters[10].Value - simParameters[9].Value,2.0);  // better to compute from other parameters

					 double FA=0.0;
					 if(SOA>=simParameters[9].Value && SOA <=simParameters[10].Value)
					 	FA = simParameters[11].Value - Math.pow(SOA-simParameters[10].Value,2.0)*k2 ;
					 else if(SOA > simParameters[10].Value)
					 	FA = simParameters[11].Value;

					// add in retouch effect
					FA *=  S1Luminance * simParameters[3].Value;

					FA *= simParameters[12].Value;

					Variables[5].Value += g1;
					Variables[6].Value += g2 + FA * (1-Variables[6].Value);


					// add in retouch cycles (S2 signal might influence)
					double PCOD = 0.0;

					if(SOA >= simParameters[13].Value && SOA <= simParameters[14].Value)
						PCOD = (SOA-simParameters[13].Value)/(simParameters[14].Value-simParameters[13].Value);
					else if (SOA > simParameters[14].Value)
						PCOD = 1.0;

					PCOD *= simParameters[15].Value;

					double ES1 = Variables[5].Value + PCOD * (1.0 - Variables[5].Value);
					double ES2 = Variables[6].Value + PCOD * (1.0 - Variables[6].Value);


					// Update percept strength
					if(simParameters[16].Value == 1.0) // S2
					{
						Variables[7].Value = ES2;
					}
					else  // S1
					{
						Variables[7].Value = ES1;
					}

                // save data: Independent Variable and Percept Strength
                ResultPair result = new ResultPair(IV, Variables[7].Value);
                ResultsSequence.addElement(result);
            }
            SimLine Line = new SimLine(conditionName, ResultsSequence);
            Results.addLine(Line);
        }

        public double EPSP(double Intensity, double a, double time)
        {
			double value = time/Math.pow(a,2.0) * Math.exp(-time/a) * Math.pow(Intensity,2.0);

			return(value);
		}
}
