// Models of backward masking -- July 2002
// Greg Francis (gfrancis@psych.purdue.edu)
// Purdue University
// This material is based upon work supported by the National Science Foundation under Grant No. 0108905. 

// Input specifies data to be plotted with different lines and symbols for different
// data sets. A legend is optionally provided.

import java.awt.image.*;
import java.awt.*;
import java.util.Date;
import java.util.Vector;

public class SimPlotImage extends Canvas
{
	double[][] ydata;
	double [][] xdata;
	double xmax, ymax, xmin, ymin;
	int num_lines;
	int num_data[];
	String xlabel, ylabel;
	String name_data[];
	int plotWidth=600, plotHeight=400;
	int offset_x, offset_y;
	boolean legend;
	String LabName;

	int numXtics=5, numYtics=5;

	Image ylabelImage=null;
	int ylabelHeight=0;
	Date date;

	// Constructor when max and min values are derived from data
	public SimPlotImage (SimResults Results)
	{
                xlabel = Results.Xaxis;
                ylabel = Results.Yaxis;              
        
                // organize data arrays
		num_lines = Results.Lines.size();
                ydata = new double [num_lines][];
                xdata = new double [num_lines][];
                num_data = new int[num_lines];
                name_data = new String[num_lines];

                // set up points
                
                for(int i=0;i<num_lines;i++)
                {
                    SimLine Line = (SimLine)Results.Lines.elementAt(i);
                    
                    // first the number of data points per condition
                    num_data[i] = Line.Points.size();
                    name_data[i] = Line.Name;
                    ydata[i] = new double [num_data[i]];
                    xdata[i] = new double [num_data[i]];
                    
                    for(int j=0;j<num_data[i];j++)
                    {
                        ResultPair result = (ResultPair)Line.Points.elementAt(j);
                        
                        ydata[i][j] = result.y;
                        xdata[i][j] = result.x;
                    }
                    
                }            
                             
		legend = true;
		this.date = Results.date;
		this.LabName = Results.Name;

		// find maximum and minimum values of data
                xmin = Double.POSITIVE_INFINITY ;
                ymin = Double.POSITIVE_INFINITY ;
		int maxNumData= 0;
		for(int j=0;j<num_lines;j++)
		{
			for(int i=0;i<num_data[j];i++)
			{
				if(this.xmax < xdata[j][i])
					this.xmax = xdata[j][i];
				if (this.xmin > xdata[j][i])
					this.xmin = xdata[j][i];
				if(this.ymax < ydata[j][i])
					this.ymax = ydata[j][i];
				if(this.ymin > ydata[j][i])
					this.ymin = ydata[j][i];
			}
			if(num_data[j]>maxNumData)
				maxNumData = num_data[j];
		}
		numXtics = (int)Math.min(numXtics, Math.max(2,maxNumData-1) );
		numYtics = (int)Math.min(numYtics, Math.max(2,maxNumData-1) );
                                
              //  MakePlot();
	}

	public void paint(Graphics g)
	{
            MakePlot(g);
	}

	// Draw the plot
	public Image MakePlot (Graphics g)
	{
		Image plotImage = createImage(plotWidth,plotHeight);
		if(ylabelImage==null)
			MakeYLabel(ylabel);

		Font font=new Font("Helvetica",Font.PLAIN,14);
		g.setFont(font);

		FontMetrics fm = getFontMetrics(font);
		int fHeight = fm.getHeight();
		int fWidth =0;

		// draw white background
		g.setColor(Color.white);
		g.fillRect(0,0,plotWidth,plotHeight);

		// define margins to box for plot (text is drawn in margins)
		int top=20, bottom=60, left=15+ylabelHeight,right=20;

		if(legend)
			right=150;
		int box_width= plotWidth-(left+right), box_height = plotHeight-(top+bottom);

		// define offset in plot so max and min values are not on box lines
		int box_offset = 15;
		g.setColor(Color.black);
		g.drawRect(left,top,box_width,box_height);

		// draw axis labels
		g.drawString(xlabel,left +box_width/2 - xlabel.length()/2*5,plotHeight-20);
		g.drawImage(ylabelImage, 2, (box_height - ylabelHeight)/2, this);

		// draw markers	for y-axis
		// divide range into up to numYtics spaces and mark them
		double yrange = (ymax - ymin)/numYtics;
		for(int i=0;i<=numYtics;i++)
		{
			double mark = (yrange*(double)i);
                        g.drawLine(left,top+box_height-box_offset-(int)(mark/(double)(ymax-ymin)*(double)(box_height-2*box_offset)), left+10, top+box_height-box_offset-(int)(mark/(double)(ymax-ymin)*(double)(box_height-2*box_offset)));

                        String label = ""+(float)(ymin+mark);
                        fWidth = fm.stringWidth(label);
                        g.drawString(label, (int)Math.max(0,left-5 - fWidth), top+box_height- box_offset-(int)(mark/(double)(ymax-ymin)*(double)(box_height-2*box_offset))+5);
		}

		// draw markers	for x-axis
		// divide range into up to numXtics spaces and mark them
		double xrange = (double)(xmax - xmin)/numXtics;
		for(int i=0;i<=numXtics;i++)
		{
			double mark = ((double)xrange*(double)i);
                        g.drawLine(left+box_offset+(int)(mark/(double)(xmax-xmin)*
					(double)(box_width-2*box_offset)),
					top+box_height-10,
					left+box_offset+(int)(mark/(double)(xmax-xmin)*
					(double)(box_width-2*box_offset)),
					top+box_height);
                        g.drawString(""+(float)(xmin+mark),left+box_offset+(int)(mark/(double)(xmax-xmin)*
					(double)(box_width-2*box_offset))-5,top+box_height+20);
		}


		// Plot each line in succession
		for(int j=0;j<num_lines;j++)
		{
			// set up data for line
			int xPts[] = new int[num_data[j]];
			int yPts[] = new int[num_data[j]];
			int	dot_size=8;
			for(int i=0;i<num_data[j];i++)
			{
				xPts[i] = left + box_offset+(int)(((double)(xdata[j][i]-xmin))/(double)(xmax-xmin)*
					(double)(box_width-2*box_offset));
				yPts[i] = top + box_offset + (int)(((ymax-ymin)-(double)(ydata[j][i]-ymin))/(ymax-ymin)*
					(double)(box_height-2*box_offset));
			}


                        // change color for different lines
                        int colorInt = j/4;
                        Color dotColor;
                        switch(colorInt)
                        {
                        case 0: 
                            dotColor = Color.black;
                            break;
                        case 1:
                            dotColor = Color.red;
                            break;
                        case 2:
                            dotColor = Color.green;
                            break;
                        case 3:
                            dotColor = Color.blue;
                            break;   
                        case 4:
                            dotColor = Color.cyan;
                            break;   
                        case 5: 
                            dotColor = Color.orange;
                            break;
                        case 6: 
                            dotColor = Color.magenta;
                            break;  
                        case 7: 
                            dotColor = Color.pink;
                            break;   
                        case 8: 
                            dotColor = Color.yellow;
                            break;                                                            
                        default:
                            dotColor = Color.black;
                        }
                        g.setColor(dotColor);
                                    
			//draw connecting lines
			for(int i=1;i<num_data[j];i++)
			{
				g.drawLine(xPts[i-1], yPts[i-1], xPts[i], yPts[i]);
			}

			// Draw symbols
                        int symbolInt = j%4;
			for(int i=0;i<num_data[j];i++)
			{
				// draw data points
				// change symbol for different lines
				switch(symbolInt)
				{
				case 0:	   // filled circle
					g.fillOval(xPts[i]-dot_size/2,yPts[i]-dot_size/2,dot_size,dot_size);
					break;
				case 1:		// filled square
					g.fillRect(xPts[i]-dot_size/2,yPts[i]-dot_size/2,dot_size,dot_size);
					break;
				case 2:		// open circle
					g.setColor(Color.white);
					g.fillOval(xPts[i]-(dot_size)/2,yPts[i]-(dot_size)/2,dot_size,dot_size);
					g.setColor(dotColor);
					g.drawOval(xPts[i]-dot_size/2,yPts[i]-dot_size/2,dot_size,dot_size);
					break;
				case 3:		// open square
					g.setColor(Color.white);
					g.fillRect(xPts[i]-(dot_size)/2,yPts[i]-(dot_size)/2,dot_size,dot_size);
					g.setColor(dotColor);
					g.drawRect(xPts[i]-dot_size/2,yPts[i]-dot_size/2,dot_size,dot_size); 
				}
			}

			// Draw legend

			if(legend)
			{
				// draw data name
				g.drawString(name_data[j],plotWidth-100, top+j*30+10);
				// draw symbol
				// change symbol for different data sets
				switch(symbolInt)
				{
				case 0:
					g.fillOval(plotWidth-130,top+j*30,dot_size,dot_size);
					break;
				case 1:
					g.fillRect(plotWidth-130,top+j*30,dot_size,dot_size);
					break;
				case 2:
					g.drawOval(plotWidth-130,top+j*30,dot_size,dot_size);
					break;
				case 3:
					g.drawRect(plotWidth-130,top+j*30,dot_size,dot_size);
				}

			}


		}
                //Draw title
                g.setColor(Color.black);
                g.drawString(LabName+" "+date.toString(), left, fHeight+1);
		return(plotImage);
	}


	public void MakeYLabel(String ylabel)
	{
		Font font=new Font("Helvetica",Font.PLAIN,14);
		FontMetrics fm = getFontMetrics(font);
		int fHeight = fm.getHeight() + fm.getMaxDescent();
		int fWidth = fm.stringWidth(ylabel);

		ylabelImage = createImage(fWidth,fHeight);

		ylabelHeight = fWidth;

		Graphics gdot = ylabelImage.getGraphics();

		gdot.setFont(font);
		gdot.setColor(Color.white);
		gdot.fillRect(0,0,fWidth,fHeight);

		gdot.setColor(Color.black);
		gdot.drawString(ylabel, 0,fm.getHeight()-1);

		// now rotate text
		try
		{
			// Get pixels of image
			PixelGrabber grabberU = new PixelGrabber(ylabelImage, 0, 0, -1, -1, true);

			if(grabberU.grabPixels())
			{
				int imageWidth = grabberU.getWidth();
				int imageHeight = grabberU.getHeight();

				int [] pixelsU = (int[])grabberU.getPixels();
				int[] pixelsNew = new int[pixelsU.length];
				// rotate image
				for(int i=0;i<imageWidth;i++)
				for(int j=0;j<imageHeight;j++)
				{
					pixelsNew[(imageWidth-1-i)*imageHeight+j] = pixelsU[j*imageWidth+i];
				}

				ylabelImage = createImage(new MemoryImageSource(imageHeight, imageWidth, pixelsNew, 0, imageHeight));
			}
		}
		catch(InterruptedException e){}
	}




}