// Models of backward masking -- July 2002
// Greg Francis (gfrancis@psych.purdue.edu)
// Purdue University
// This material is based upon work supported by the National Science Foundation under Grant No. 0108905.

// This  class implements the simple mask blocking model of Francis & Cho (2004).
// June 2004
import java.io.*;
import java.util.Vector;

public class SimpleMaskBlocking extends Simulation implements Serializable
{
    public SimpleMaskBlocking()
    {
        Name = "Simple mask-blocking model";
        Description = "This is a simulation of the simple mask-blocking system first described by Francis & Cho (2004). ";

        Results = new SimResults(Name, "SOA (ms)", "Percept strength");

        // define Independent Variable parameters
        IVParameters = new Parameter[4];
        IVParameters[0] = new Parameter("Independent variable", 6, "This indicates which parameter is to be used as the independent variable. Pick an integer that corresponds to one of the model parameters.");
        IVParameters[1] = new Parameter("IV_Start", 0.0, "IV_Start is the starting value of the independent variable.");
        IVParameters[2] = new Parameter("IV_End", 100.0, "IV_End is the ending value of the independent variable.");
        IVParameters[3] = new Parameter("IV_Step", 5.0, "IV_Step is the size of steps through the independent variable.");

        // define simulation parameters
        simParameters = new Parameter[8];
        simParameters[0] = new Parameter("A", 0.1, "A sets the rate of decay of the target and mask initial responses.");
        simParameters[1] = new Parameter("G", 0.1, "G is a threshold. To compute percept strength the target VRF is computed up to the time that the VRF goes below the value of G. Setting G smaller will tend to make the percept strength larger.");
        simParameters[2] = new Parameter("Target intensity", 10, "Target intensity is the strength of the target input.");
        simParameters[3] = new Parameter("Target duration", 10, "Target duration is the duration of the target input.");
        simParameters[4] = new Parameter("Mask intensity", 5, "Mask intensity is the strength of the mask input.");
        simParameters[5] = new Parameter("Mask duration", 10, "Mask duration is the duration of the mask input.");
        simParameters[6] = new Parameter("SOA", 0.0, "SOA is the stimulus onset asynchrony between the target and the mask. This is the time between the onset of the target and the onset of the mask.");
        simParameters[7] = new Parameter("DeltaT", 0.001, "DeltaT is the temporal resolution of the simulation. Variables are checked once every DeltaT time steps. Smaller values will cause the simulation to take longer. Smaller values also produce more accurate calculations of the differential equations.");

        // define Variables
        Variables = new Parameter[4];
        Variables[0] = new Parameter("Target response", 0.0, "The target response is a visual system's initial response to the target stimulus. Percept strength is the integral of the target VRF.");
        Variables[1] = new Parameter("Percept strength", 0.0, "The percept strength is a representation of the visiblity of the target. Larger values indicate better visibility.");
        Variables[2] = new Parameter("Mask response", 0.0, "The mask response is a visual system's initial response to the mask stimulus. ");
		Variables[3] = new Parameter("Target VRF", 0.0, "The mask VRF is a visual system's response to the target and mask stimulus. ");

        // Save these as defaults
        IVParametersDefault = new Parameter[IVParameters.length];
        for(int i=0;i<IVParameters.length;i++)
            IVParametersDefault[i] = IVParameters[i].copy();
        simParametersDefault = new Parameter[simParameters.length];
        for(int i=0;i<simParameters.length;i++)
            simParametersDefault[i] = simParameters[i].copy();
        variablesDefault = new Parameter[Variables.length];
        for(int i=0;i<Variables.length;i++)
            variablesDefault[i] = Variables[i].copy();
    }

	public static void main(String args[])
	{
		SimpleMaskBlocking simulation = new SimpleMaskBlocking();
                new SimulationGUI(simulation);
	}

        public boolean  UpdateParameters(Parameter[] newIVParameters, Parameter[] newSimParameters, Parameter[] newOutputParameters)
        {
            for(int i=0;i<IVParameters.length;i++)
                IVParameters[i].Value = newIVParameters[i].Value;
            for(int i=0;i<simParameters.length;i++)
                simParameters[i].Value = newSimParameters[i].Value;

            return(true);
        }

	public boolean CheckParameters()
        {
            // not implemented
            return(true);
        }

    public void start()
    {
        done=false;
        animate = new Thread(this);
        animate.setPriority(5);
        animate.start();
    }

    public void run()
    {
        RunSimulation(this.CurrentRunName);
        done = true;
    }

    public void startRun(String conditionName)
    {
        this.CurrentRunName = conditionName;
        start();
    }

	public void RunSimulation(String conditionName)
        {
            Vector ResultsSequence = new Vector();

            int numberOfRuns = (int)((IVParameters[2].Value - IVParameters[1].Value)/IVParameters[3].Value);
            int numOfRunsSoFar=0;
            for(double IV = IVParameters[1].Value; IV <= IVParameters[2].Value; IV+= IVParameters[3].Value )
            {
                int percent = (int) ( 100*((float)numOfRunsSoFar/(float)numberOfRuns));
                numOfRunsSoFar++;
                Status = ""+percent;
                // set IV appropriately
                simParameters[(int)IVParameters[0].Value].Value = IV;

                // initialize cell activities to zero
                for(int i=0;i<Variables.length;i++)
                    Variables[i].Value = 0.0;

                // now compute percept strength for this condition
                double TargetLuminance = 0.0, MaskLuminance = 0.0;

                // find start time, needs to be negative value if SOA is negative
                double time = 0.0;
                if(simParameters[6].Value < 0)
                    time = simParameters[6].Value;
                boolean beenAboveThreshold=false;
                boolean stopSimulation=false;
                double OnTime=0;
                while(!stopSimulation)
                {
                    // Set up intensities of target for this time step
                    if(time>=0.0 && time<simParameters[3].Value)
                        TargetLuminance = simParameters[2].Value;
                    else
                        TargetLuminance=0.0;

                    // Set up intensities of mask for this time step
                    if(time>=simParameters[6].Value && time < simParameters[6].Value + simParameters[5].Value)
                        MaskLuminance=simParameters[4].Value;
                    else
                        MaskLuminance=0.0;

                    // Update target response
                    Variables[0].Value += simParameters[7].Value*(-simParameters[0].Value * Variables[0].Value +  TargetLuminance);

  	                 // Update mask response
                    Variables[2].Value += simParameters[7].Value*(-simParameters[0].Value * Variables[2].Value +  MaskLuminance );

					// Update target VRF
                    Variables[3].Value += simParameters[7].Value*(-simParameters[0].Value * Variables[3].Value +  Variables[0].Value - Math.max(Variables[2].Value - Variables[0].Value,0.0));

					 // - Variables[2].Value/( Variables[0].Value + 0.01) );

					//Math.max(Variables[2].Value - Variables[0].Value,0.0));

					// Update percept strength
					Variables[1].Value += simParameters[7].Value * Math.max(Variables[3].Value - simParameters[1].Value,0.0);


                    // note OnTime, when VRF goes above threshold
                    if(!beenAboveThreshold)
                    {
                        if(Variables[3].Value  > simParameters[1].Value)
                        {
                            OnTime=time;
                            beenAboveThreshold=true;
                        }
                    }

                    // If VRF never goes above threshold, percept strength is zero
                    if(time>simParameters[3].Value && !beenAboveThreshold)
                    {
                        stopSimulation=true;
                        Variables[1].Value=0;
                    }

                    // note stop time, when VRF goes below threshold
                    if(beenAboveThreshold && Variables[3].Value < simParameters[1].Value)
                    {
                        stopSimulation=true;
                    }
                    // update time
                    time+= simParameters[7].Value;
                }

                // save data: Independent Variable and Percept Strength
                ResultPair result = new ResultPair(IV, Variables[1].Value);
                ResultsSequence.addElement(result);
            }
            SimLine Line = new SimLine(conditionName, ResultsSequence);
            Results.addLine(Line);
        }
}
