// Models of backward masking -- July 2002
// Greg Francis (gfrancis@psych.purdue.edu)
// Purdue University
// This material is based upon work supported by the National Science Foundation under Grant No. 0108905. 

// This  class implements the masking model of Weisstein (1972).

// Revised 17 December 2002: fixed a bug with the threshold check for stopping the simulation. It turns out that the variable ofinterest bounces, it can go below and then rise back up above the threshold. The visibility calculation was missing the second part of the above threshold activity. 

import java.io.*;
import java.util.Vector;

public class Weisstein extends Simulation implements Serializable
{        
    public Weisstein()
    {
        Name = "Dual-channel model";
        Description = "This is a simulation of the dual-channel model described by Weisstein (1972). The starting parameters and stimulus characteristics are those used in Francis (2000).";

        Results = new SimResults(Name, "SOA (ms)", "Percept strength");          
        // define Independent Variable parameters
        IVParameters = new Parameter[4];
        IVParameters[0] = new Parameter("Independent variable", 25, "This indicates which parameter is to be used as the independent variable. Pick an integer that corresponds to one of the model parameters."); 
        IVParameters[1] = new Parameter("IV_Start", -50.0, "IV_Start is the starting value of the independent variable.");
        IVParameters[2] = new Parameter("IV_End", 250.0, "IV_End is the ending value of the independent variable.");
        IVParameters[3] = new Parameter("IV_Step", 10.0, "IV_Step is the size of steps through the independent variable."); 
		
        // define simulation parameters
        simParameters = new Parameter[28];
        
        // Target pathway
        // first cell
        simParameters[0] = new Parameter("A1-T", 1.5, "A1-T scales the input feeding in to the depolarizing component of the first cell in the target pathway.");
        simParameters[1] = new Parameter("a1-T", 0.025, "a1-T sets the rate of passive decay for the depolarizing component of the first cell in the target pathway.");
        simParameters[2] = new Parameter("B1-T", 1.5, "B1-T scales the input feeding in to the hyperpolarizing component of the first cell in the target pathway.");
        simParameters[3] = new Parameter("b1-T", 0.1, "b1-T sets the rate of passive decay for the depolarizing component of the first cell in the target pathway.");     
        
        // second cell
        simParameters[4] = new Parameter("A2-T", 1.2, "A2-T scales the input feeding in to the depolarizing component of the second cell in the target pathway.");
        simParameters[5] = new Parameter("a2-T", 0.03, "a2-T sets the rate of passive decay for the depolarizing component of the second cell in the target pathway.");
        simParameters[6] = new Parameter("B2-T", 2.4, "B2-T scales the input feeding in to the hyperpolarizing component of the second cell in the target pathway.");
        simParameters[7] = new Parameter("b2-T", 0.09, "b2-T sets the rate of passive decay for the depolarizing component of the second cell in the target pathway.");              
                 
        // third cell
        simParameters[8] = new Parameter("A3-T", 0.2, "A3-T scales the input feeding in to the depolarizing component of the third cell in the target pathway.");
        simParameters[9] = new Parameter("a3-T", 0.009, "a3-T sets the rate of passive decay for the depolarizing component of the third cell in the target pathway.");
        simParameters[10] = new Parameter("B3-T", 0.0, "B3-T scales the input feeding in to the hyperpolarizing component of the third cell in the target pathway.");
        simParameters[11] = new Parameter("b3-T", 0.0, "b3-T sets the rate of passive decay for the depolarizing component of the third cell in the target pathway.");         
 
        // Mask pathway
        // first cell
        simParameters[12] = new Parameter("A1-M", 1.5, "A1-M scales the input feeding in to the depolarizing component of the first cell in the mask pathway.");
        simParameters[13] = new Parameter("a1-M", 0.025, "a1-M sets the rate of passive decay for the depolarizing component of the first cell in the mask pathway.");
        simParameters[14] = new Parameter("B1-M", 1.5, "B1-M scales the input feeding in to the hyperpolarizing component of the first cell in the mask pathway.");
        simParameters[15] = new Parameter("b1-M", 0.1, "b1-M sets the rate of passive decay for the depolarizing component of the first cell in the mask pathway.");           
        
         // second cell
        simParameters[16] = new Parameter("A2-M", 1.6, "A2-M scales the input feeding in to the depolarizing component of the second cell in the mask pathway.");
        simParameters[17] = new Parameter("a2-M", 0.3, "a2-M sets the rate of passive decay for the depolarizing component of the second cell in the mask pathway.");
        simParameters[18] = new Parameter("B2-M", 6.4, "B2-M scales the input feeding in to the hyperpolarizing component of the second cell in the mask pathway.");
        simParameters[19] = new Parameter("b2-M", 0.6, "b2-M sets the rate of passive decay for the depolarizing component of the second cell in the mask pathway.");  
        
        // Stimulus parameters
        simParameters[20] = new Parameter("G", 1000.0, "G is a threshold. It is used only when the percept calculation parameter is set to '2'.  To compute percept strength the target VRF is integrated up to the time that the VRF goes below the value of G. Setting G smaller will tend to make the percept strength larger.");       
        simParameters[21] = new Parameter("Target intensity", (3.75 * Math.log(10)+26.0), "Target intensity is the strength of the target input.");
        simParameters[22] = new Parameter("Target duration", 20.0, "Target duration is the duration of the target input.");
        simParameters[23] = new Parameter("Mask intensity", (3.75 * Math.log(10)+26.0), "Mask intensity is the strength of the mask input.");
        simParameters[24] = new Parameter("Mask duration", 20.0, "Mask duration is the duration of the mask input.");

        simParameters[25] = new Parameter("SOA", 0.0, "SOA is the stimulus onset asynchrony between the target and the mask. This is the time between the onset of the target and the onset of the mask.");
        simParameters[26] = new Parameter("DeltaT", 0.01, "DeltaT is the temporal resolution of the simulation. Variables are checked once every DeltaT time steps. Smaller values will cause the simulation to take longer. Smaller values also make for more precise calculations of the differential equations."); 
        
        // Percept strength calculation parameter
        simParameters[27] = new Parameter("Percept calculation", 2, "Weisstein's model can use one of three calculations of the strength of the target percept. You can select which version you want to use.\n\n 0 (maximal activity): in this version the maximum strength of the third (last) cell in the target pathway is the strength of the target percept. This version was apparently used by Weisstein (1972) to produce her Figure 2.\n\n 1 (integration to 400): in this version, the integral of the activty of the third (last) cell in the target pathway is taken from the onset of the target to time 400. A mathematical description of this version is given in Weisstein (1972, pp. 248-249). There is some evidence that this version is what was used for Weisstein's fits to experimental data. \n \n 2 (integration above a threshold):  in this version the integral of activity of the third (last) cell in the target pathway is integrated up to the time that the activity is above a given threshold (set by parameter G). This version is perhaps the most faithful to the verbal description Weisstein (1972) gave to her model, and was proposed by Francis (2000). ");
        

        // define Variables
        Variables = new Parameter[11];
        
        //target
        Variables[0] = new Parameter("et-1", 0.0, "The excitatory part of the first cell in the target pathway.");
        Variables[1] = new Parameter("jt-1", 0.0, "The inhibitory part of the first cell in the target pathway.");
        Variables[2] = new Parameter("et-2", 0.0, "The excitatory part of the second cell in the target pathway.");
        Variables[3] = new Parameter("jt-2", 0.0, "The inhibitory part of the second cell in the target pathway.");
        Variables[4] = new Parameter("et-3", 0.0, "The excitatory part of the third cell in the target pathway.");
        Variables[5] = new Parameter("jt-3", 0.0, "The inhibitory part of the third cell in the target pathway."); 
        
        // mask       
        Variables[6] = new Parameter("em-1", 0.0, "The excitatory part of the first cell in the mask pathway.");
        Variables[7] = new Parameter("jm-1", 0.0, "The inhibitory part of the first cell in the mask pathway.");     
        Variables[8] = new Parameter("em-2", 0.0, "The excitatory part of the second cell in the mask pathway.");
        Variables[9] = new Parameter("jm-2", 0.0, "The inhibitory part of the second cell in the mask pathway.");        
        // Percept strength
        Variables[10] = new Parameter("Percept strength", 0.0, "The percept strength is a representation of the visiblity of the target. Larger values indicate better visibility.");
                                        
        // Save these as defaults
        IVParametersDefault = new Parameter[IVParameters.length];
        for(int i=0;i<IVParameters.length;i++)
            IVParametersDefault[i] = IVParameters[i].copy();
        simParametersDefault = new Parameter[simParameters.length];
        for(int i=0;i<simParameters.length;i++)
            simParametersDefault[i] = simParameters[i].copy();       
        variablesDefault = new Parameter[Variables.length];
        for(int i=0;i<Variables.length;i++)
            variablesDefault[i] = Variables[i].copy();   
    }
    
	public static void main(String args[]) 
	{
		Weisstein simulation = new Weisstein();
                new SimulationGUI(simulation);
	}    

        public boolean  UpdateParameters(Parameter[] newIVParameters, Parameter[] newSimParameters, Parameter[] newOutputParameters)
        {
            for(int i=0;i<IVParameters.length;i++)
                IVParameters[i].Value = newIVParameters[i].Value;           
            for(int i=0;i<simParameters.length;i++)
                simParameters[i].Value = newSimParameters[i].Value;     
            return(true);
        }

	public boolean CheckParameters()
        {
            // not implemented
            return(true);
        }

    public void start()
    {
        done=false;
        animate = new Thread(this);
        animate.setPriority(5);
        animate.start();
    }

    public void run()
    {
        RunSimulation(this.CurrentRunName);
        done = true;
    }
    
    public void startRun(String conditionName)
    {
        this.CurrentRunName = conditionName;
        start();
    }
    
	public void RunSimulation(String conditionName)
        {
            Vector ResultsSequence = new Vector();
            
            int numberOfRuns = (int)((IVParameters[2].Value - IVParameters[1].Value)/IVParameters[3].Value);
            int numOfRunsSoFar=0;
            for(double IV = IVParameters[1].Value; IV <= IVParameters[2].Value; IV+= IVParameters[3].Value )
            {
                
                int percent = (int) ( 100*((float)numOfRunsSoFar/(float)numberOfRuns));
                numOfRunsSoFar++;
                Status = ""+percent;
                
                // set IV appropriately
                simParameters[(int)IVParameters[0].Value].Value = IV;

                // initialize cell activities to zero
                for(int i=0;i<Variables.length;i++)
                    Variables[i].Value = 0.0;

                // now compute percept strength for this condition
                double TargetLuminance = 0.0, MaskLuminance = 0.0;
                
                // find start time, needs to be negative value if SOA is negative
                double startTime = 0.0;
                if(simParameters[25].Value < 0)
                    startTime = simParameters[25].Value;
                boolean beenAboveThreshold=false;
                // find end time, if using percept calculation method 1, it can be very short
                double endTime = 20000;
                if(simParameters[27].Value ==1)
                {
                    // make certain threshold is zero
                    simParameters[20].Value = 0.0;
                    endTime = 400.0;
                }
                
                for(double time=startTime;time<endTime;time+=simParameters[26].Value)
		{
                    // Set up intensities of target for this time step
                    if(time>=0.0 && time<simParameters[22].Value)
                        TargetLuminance = simParameters[21].Value;
                    else
                        TargetLuminance=0.0;
  
                    // Set up intensities of mask for this time step
                    if(time>=simParameters[25].Value && time < simParameters[25].Value + simParameters[24].Value)
                        MaskLuminance=simParameters[23].Value;
                    else
                        MaskLuminance=0.0;   
			
                    // run equations
                    // cell 1-1 (input target)
                    Variables[0].Value += simParameters[26].Value*(simParameters[0].Value*TargetLuminance - simParameters[1].Value*Variables[0].Value);
                    Variables[1].Value += simParameters[26].Value*(simParameters[2].Value*TargetLuminance - simParameters[3].Value*Variables[1].Value);
			
                    // cell 2-1 (input mask)
                    Variables[6].Value += simParameters[26].Value*(simParameters[12].Value*MaskLuminance - simParameters[13].Value*Variables[6].Value);
                    Variables[7].Value += simParameters[26].Value*(simParameters[14].Value*MaskLuminance - simParameters[15].Value*Variables[7].Value);	
			
                    // cell 1-2 (second cell for target)	
                    Variables[2].Value += simParameters[26].Value*(simParameters[4].Value*Math.max(Variables[0].Value-Variables[1].Value,0.0) - simParameters[5].Value*Variables[2].Value );
                    Variables[3].Value += simParameters[26].Value*(simParameters[6].Value*Math.max(Variables[0].Value-Variables[1].Value,0.0) - simParameters[7].Value*Variables[3].Value );
			
                    // cell 2-2 (second cell for mask)	
                    Variables[8].Value += simParameters[26].Value*(simParameters[16].Value*Math.max(Variables[6].Value-Variables[7].Value,0.0) - simParameters[17].Value*Variables[8].Value );
                    Variables[9].Value += simParameters[26].Value*(simParameters[18].Value*Math.max(Variables[6].Value-Variables[7].Value,0.0) - simParameters[19].Value*Variables[9].Value );
			
                    // cell 1-3 (third cell for target)	
                    Variables[4].Value += simParameters[26].Value*(simParameters[8].Value*((Math.max(Variables[2].Value-Variables[3].Value,0.0) - Math.max(Variables[9].Value-Variables[8].Value,0.0))) - simParameters[9].Value*Variables[4].Value );  // switched e & j, for inhibition, relative to Weisstein's paper
                    Variables[5].Value += simParameters[26].Value*(simParameters[10].Value*((Math.max(Variables[2].Value-Variables[3].Value,0.0) - Math.max(Variables[9].Value-Variables[8].Value,0.0))) - simParameters[11].Value*Variables[5].Value );  // switched e & j, for inhibition, relative to Weisstein's paper


                    // visibility - starts at onset of target
                    if(time>0)
                    {
                        if(simParameters[27].Value == 0) // visibility as maximum value
                        {
                            if(Variables[10].Value < Variables[4].Value - Variables[5].Value)
                                Variables[10].Value = Variables[4].Value - Variables[5].Value;
                        }
                        else if(simParameters[27].Value ==1) // visibility as integration to 400
                        {
                            if(time <=400)
                                Variables[10].Value += simParameters[26].Value*Math.max(Variables[4].Value - Variables[5].Value,0.0); 
                        }
                        else  // visibility as integration to a threshold value
                            Variables[10].Value += simParameters[26].Value*(Math.max(Variables[4].Value  - Variables[5].Value-simParameters[20].Value,0.0)); 
                          
                    }


                    // need to note that VRF has gone above threshold, else do not know when to stop
                    if(!beenAboveThreshold)
                    {
                        if(Math.max(Variables[4].Value-Variables[5].Value,0.0)>=simParameters[20].Value )
                        {
                            beenAboveThreshold=true;
                        }
                    }
                    
                    // stop when VRF is below threshold    
                    if(beenAboveThreshold)
                    {                
                        if(time>400 && Math.max(Variables[4].Value-Variables[5].Value,0.0)<simParameters[20].Value)
                        {
                            time = 200000;
                            beenAboveThreshold=false;
                        }
                    }
		}	                
                
                // save data: Independent Variable and Percept Strength
                ResultPair result = new ResultPair(IV, Variables[10].Value);	              
                ResultsSequence.addElement(result);
            }
            SimLine Line = new SimLine(conditionName, ResultsSequence);
            Results.addLine(Line);
        }
}
